@@ -5,7 +5,7 @@ import click
 from jinja2 import Environment, FileSystemLoader
 
 
-__version__ = "0.2.1"
+__version__ = "0.3.0"
 
 
 # Compatibility with Python 2.7
@@ -31,8 +31,8 @@ class FileCount:
         return '{}/{}/{}'.format(self.created, self.overwritten, self.skipped)
 
 
-def textx_jinja_generator(templates_path, target_path, config, overwrite=False,
-                          filters=None):
+def textx_jinja_generator(templates_path, target_path, context, overwrite=False,
+                          filters=None, transform_names=None):
     """
     Generates a set of files using Jinja templates.
     """
@@ -41,31 +41,52 @@ def textx_jinja_generator(templates_path, target_path, config, overwrite=False,
     Args:
         templates_path (str): A path to templates.
         target_path (str): The path where files should be generated.
-        config (dict): A config contains any data necessary
+        context (dict): A context contains any data necessary
             for rendering files using Jinja engine.
         overwrite (bool): If the target files should be overwritten.
         filters(dict): Optional Jinja filters.
+        transform_names(callable or None): If given, used to transform resolved
+            placeholders in file names. The function accepts a single parameter
+            and should return a string. It should default to `str(param)` in case
+            no other transformation could be provided.
+
     """
     def eval_file_name(file_name):
         """
-        Replaces all `__<var>__` if `var` is in config dict.
+        Replaces all `__<var>__` if `var` is in context dict.
         Strips .jinja extension.
         Raises SkipFile if file shouldn't be processed.
         """
         # Replace placeholders in the target file name.
         placeholders = placeholder_re.findall(file_name)
+        files = None
+        tran_names = str if transform_names is None else transform_names
 
         for placeholder in placeholders:
-            ph_value = config.get(placeholder.strip('_'))
+            ph_value = context.get(placeholder.strip('_'))
             if ph_value is False:
                 raise SkipFile
             elif ph_value is True:
                 file_name = file_name.replace(placeholder, '')
             elif ph_value is not None:
-                file_name = file_name.replace(placeholder, ph_value)
+                try:
+                    iter(ph_value)
+                    if isinstance(ph_value, str):
+                        raise TypeError
+                    files = {}
+                    for ph in ph_value:
+                        f_name = file_name.replace(placeholder, tran_names(ph))
+                        if f_name.endswith('.jinja'):
+                            f_name = '.'.join(f_name.split('.')[:-1])
+                        files[f_name] = ph
+                except TypeError:
+                    file_name = file_name.replace(placeholder, tran_names(ph_value))
 
         if file_name.endswith('.jinja'):
             file_name = '.'.join(file_name.split('.')[:-1])
+
+        if files is not None:
+            return files
         return file_name
 
     def generate_file(src_rel_file, src_file, target_file):
@@ -85,7 +106,7 @@ def textx_jinja_generator(templates_path, target_path, config, overwrite=False,
                 # Render using Jinja template
                 with open(target_file, 'w') as f:
                     f.write(
-                        env.get_template(src_rel_file).render(**config))
+                        env.get_template(src_rel_file).render(**context))
             else:
                 # Just copy
                 if not src_file == target_file:
@@ -128,6 +149,7 @@ def textx_jinja_generator(templates_path, target_path, config, overwrite=False,
             for f in files:
                 src_file = os.path.join(root, f)
                 src_rel_file = os.path.relpath(src_file, templates_path)
+                src_rel_file = src_rel_file.replace('\\', '/')
                 target_file = os.path.join(target_path, src_rel_file)
 
                 try:
@@ -136,13 +158,24 @@ def textx_jinja_generator(templates_path, target_path, config, overwrite=False,
                 except SkipFile:
                     continue
 
-                # Create necessary folders.
-                try:
-                    os.makedirs(os.path.dirname(target_file))
-                except FileExistsError:
-                    pass
+                # If target_file is dictionary generate file for each key-value pair.
+                if isinstance(target_file, dict):
+                    try:
+                        os.makedirs(os.path.dirname(list(target_file)[0]))
+                    except FileExistsError:
+                        pass
 
-                generate_file(src_rel_file, src_file, target_file)
+                    for f_name in target_file:
+                        context['obj'] = target_file[f_name]
+                        generate_file(src_rel_file, src_file, f_name)
+                else:
+                    # Create necessary folders.
+                    try:
+                        os.makedirs(os.path.dirname(target_file))
+                    except FileExistsError:
+                        pass
+
+                    generate_file(src_rel_file, src_file, target_file)
 
     click.echo('Done. Files created/overwritten/skipped = {}'
                .format(file_count))
